"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decorateMarkdown = void 0;
const slate_1 = require("slate");
const tokenizer_1 = require("./tokenizer");
function getHeadingType(token) {
    switch (token.depth) {
        case 1:
            return "h1";
        case 2:
            return "h2";
        case 3:
            return "h3";
        case 4:
            return "h4";
        case 5:
            return "h5";
        case 6:
            return "h6";
    }
}
function decorateMarkdown({ entry, options: { disabled = {} }, }) {
    const [node] = entry;
    const ranges = [];
    if (!slate_1.Text.isText(node)) {
        return ranges;
    }
    const tokens = (0, tokenizer_1.getTokens)(node.text);
    (0, tokenizer_1.walkTokens)(tokens, (token) => {
        // @ts-ignore
        if (disabled[token.type]) {
            return;
        }
        const entryRanges = getRangeByToken(entry, token);
        if (entryRanges) {
            ranges.push(entryRanges);
        }
        const markerRanges = getMarkerRangeByToken(entry, token);
        if (markerRanges) {
            ranges.push(...markerRanges);
        }
    });
    return ranges;
}
exports.decorateMarkdown = decorateMarkdown;
function getRangeByToken(entry, token) {
    const [, path] = entry;
    switch (token.type) {
        case "strong":
        case "em":
        case "del":
        case "codespan":
            return {
                [token.type]: true,
                anchor: { path, offset: token._start },
                focus: { path, offset: token._end },
            };
        case "heading": {
            const type = getHeadingType(token);
            if (type) {
                return {
                    [type]: true,
                    anchor: { path, offset: token._start },
                    focus: { path, offset: token._end },
                };
            }
        }
    }
}
function getMarkerRangeByToken(entry, token) {
    const [, path] = entry;
    switch (token.type) {
        case "strong":
        case "em":
        case "del":
        case "codespan": {
            const [prefix, suffix] = token.raw.split(token.text);
            if (prefix && suffix) {
                return [
                    {
                        marker: true,
                        anchor: { path, offset: token._start },
                        focus: { path, offset: token._start + prefix.length },
                    },
                    {
                        marker: true,
                        anchor: { path, offset: token._end - suffix.length },
                        focus: { path, offset: token._end },
                    },
                ];
            }
            if (prefix) {
                return [
                    {
                        marker: true,
                        anchor: { path, offset: token._start },
                        focus: { path, offset: token._start + prefix.length },
                    },
                ];
            }
            if (suffix) {
                return [
                    {
                        marker: true,
                        anchor: { path, offset: token._end - suffix.length },
                        focus: { path, offset: token._end },
                    },
                ];
            }
            break;
        }
        case "link": {
            return [
                {
                    marker: true,
                    anchor: { path, offset: token._start },
                    focus: { path, offset: token._start + 1 },
                },
                {
                    marker: true,
                    anchor: { path, offset: token._end - token.href.length + 3 },
                    focus: { path, offset: token._end },
                },
            ];
        }
        case "list_item": {
            const match = token.raw.match(tokenizer_1.rules.listItemStart);
            if (match) {
                return [
                    {
                        marker: true,
                        anchor: { path, offset: token._start },
                        focus: { path, offset: token._start + match[0].length },
                    },
                ];
            }
            break;
        }
        case "heading": {
            const type = getHeadingType(token);
            if (type) {
                return [
                    {
                        marker: true,
                        anchor: { path, offset: token._start },
                        focus: { path, offset: token._start + token.depth },
                    },
                ];
            }
        }
    }
}
